 #
 #   $Id: Blog.pm,v 1.2 2006/07/08 19:10:39 zarahg Exp $
 #
 #   This  file  is  part of the  PodWiki  web-based authoring tool.
 #
 #   By  accessing  this software,  PodWiki,  you are  duly informed
 #   of and  agree to be  bound  by the  conditions  described below
 #   in this notice:
 #
 #   This software product, PodWiki,  is developed by  Thomas Linden
 #   and   copyrighted  (C) 2003-2005   by  Thomas Linden,  with all
 #   rights reserved.
 #
 #   There is  no charge for  PodWiki software. You can redistribute
 #   it and/or modify it under  the terms of the  GNU General Public
 #   License, which is incorporated by reference herein.
 #
 #   PodWiki is distributed WITHOUT ANY WARRANTY,IMPLIED OR EXPRESS,
 #   OF  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that
 #   the  use of it will not infringe on any third party's intellec-
 #   tual property rights.
 #
 #   You  should  have  received a  copy  of  the GNU General Public
 #   License along with PodWiki.  Copies  can also be obtained from:
 #
 #     http://www.gnu.org/copyleft/gpl.html
 #
 #   or by writing to:
 #
 #     Free Software Foundation, Inc.
 #     59 Temple Place, Suite 330
 #     Boston, MA 02111-1307
 #     USA
 #
 #   Or contact:
 #
 #    "Thomas Linden" <tom@daemon.de>
 #
 #   Additional Copyrights:
 #
 #    Copyright 2004 by David Bright
 #

package PodWiki::Formatter::Blog;

use PodWiki::Revision;

$PodWiki::Formatter::Blog::VERSION = "1.00";

use vars qw(@ISA @EXPORT @EXPORT_OK @EXPORT_TAGS @docpages);
use vars qw($me $config);

@ISA=qw(Exporter PodWiki::Runner PodWiki::Formatter);
@EXPORT=qw();
@EXPORT_OK=qw();
@EXPORT_TAGS=();

sub new {
  my($this, %param) = @_;
  my $class = ref($this) || $this;
  my $self = \%param;
  $config = $self->{config};
  bless($self, $class);
  $self->{BlogCategory} = "BLOG";
  $self->{BlogUser}     = "root";
  $self->{BlogPostings} = 10;
  return $self;
}

sub tags {
  my($this) = @_;
  return qw(blog);
}

sub parse_page {
  my($this, $text) = @_;
  local $_ = $text;

  # the blog formatter considers its content as config settings

  s/#.*$//gs;     # remove trailing comments

  foreach (split /\n/) {
    next if (/^#/);
    s/\s\s*$//;
    s/^\s\s*//;
    if (/category\s*=\s*(.*)\s*$/) {
      $this->{BlogCategory} = $1;
    }
    elsif (/user\s*=\s*(.*)$/) {
      $this->{BlogUser} = $1;
    }
    elsif (/postings\s*=\s*(.*)$/) {
      $this->{BlogPostings} = $1;
    }
  }
}

sub format {
  my($this, $text) = @_;
  $this->parse_page($text);
  $this->show_admin();
  $this->show_postings();
}


sub show_feed {
   my ($this) = @_;
   my $input = $PodWiki::Runner::input;
   my $page  = $input->{blog};
   $this->parse_page(PodWiki::Runner::fetch_page($page));
   $this->show_postings("xml");
}


sub show_admin {
  my($this) = @_;
  if ($PodWiki::Runner::current_user eq $this->{BlogUser}) {
     print qq(<div class="blogadmin"><h4>Menu for $PodWiki::Runner::page</h4><i>Logged in as $this->{BlogUser}</i>
     <p><a href="$PodWiki::Runner::me?state=edit&page=ReplaceTitlePlease&inherit=$PodWiki::Runner::page">Create new blog posting</a></p>
     <form title="change_$this->{BlogCategory}" action="$PodWiki::Runner::me" method="post">
      <input type="hidden" name="state" value="edit_attribute_admin">
      <input type="hidden" name="nextstate" value="render_page">
      <input type="hidden" name="page" value="$PodWiki::Runner::page">
      <input type="hidden" name="attribute" value="$this->{BlogCategory}">
      Maintain categories (separated by whitespace):<br/> 
      <input type="text" name="attr_values" size="60" value="$PodWiki::Runner::attributes->{$this->{BlogCategory}}">
      <input type="submit" name="submit" value="change">
     </div> 
     );
  }
}



sub show_postings {
  my($this, $xml) = @_;
  my %pages;
  if (exists $PodWiki::Runner::attributes->{".classified"}->{$this->{BlogCategory}}) {
    foreach my $value (keys %{$PodWiki::Runner::attributes->{".classified"}->{$this->{BlogCategory}}}) {
      foreach my $page (sort keys %{$PodWiki::Runner::attributes->{".classified"}->{$this->{BlogCategory}}->{$value}}) {
	if ($page eq $PodWiki::Runner::page) {
            # do not list myself
	    next;
	}
        if (-e "$PodWiki::Runner::config->{data}/$page") {
           my $rcs = new PodWiki::Revision(@PodWiki::Runner::rcs, "$page");
	   my $revision = $rcs->head;
	   my $time = $rcs->revdate($revision);
           $pages{$time}->{page} = "$page";
	   $pages{$time}->{cat}  = "$value"; 
	   $pages{$time}->{author} = $rcs->author($revision);
	   $pages{$time}->{revision} = $revision;   
        }
      }
    }
  }

  my $max = $this->{BlogPostings};
 
  if (! $xml) {
      print qq(<div class="blog">);
  }

  foreach my $time (reverse sort keys %pages) {
      last if (! $max);
      $max--;
      my $page = $pages{$time}->{page};
      my $cat  = $pages{$time}->{cat};
      my $author = $pages{$time}->{author};
      my $revision = $pages{$time}->{revision};
      my $content = PodWiki::Runner::fetch_page($page);
      if ($xml) {
        print qq(<item><title>$page</title>
	         <link>$PodWiki::Runner::me?page=$page</link>
		 <pubDate>) . scalar localtime($time) . qq(</pubDate>
		 <dc:creator>$author</dc:creator>
		 <category>$cat</category>
		 <guid>$PodWiki::Runner::me?page=$page</guid>
		 <description><content:encoded><![CDATA[\n);
	PodWiki::Runner::render(PodWiki::Runner::fetch_page($page));
	print qq(\n]]></content:encoded></description></item>); 
      }
      else {
        print qq(<div class="blogtitle"><i>) 
	 .PodWiki::Runner->pod_link($page)
	 .qq(</i></div><div class="blogpost">);

        if ($content =~ /(.+?)(N<([^>]*)>).*$/s) {
	    my $show = $1;
	    my $cap  = $3;
	    my $name = $page;
	    $name =~  s/\s/%20/g;
	    $content = $show . "L<$cap|$name>\n\n=cut\n";
	}
        PodWiki::Runner::render($content);
      
        print "</div><br/>\n";
        print qq(<div class="blogfoot">posted by <i>$author</i>, revision <i>$revision</i>, on <i>) . scalar localtime($time) . qq(</i>
	  in <a href="$PodWiki::Runner::me?state=dosearch&podattributes-$this->{BlogCategory}=$cat&submit-attributes=search">$cat</a> 
	  );
        if ($PodWiki::Runner::current_user eq $this->{BlogUser}) {
          print qq((<a href="$PodWiki::Runner::me?state=edit&page=$page">edit</a>));
        }
	PodWiki::Runner::tie_comment();
	if (exists $PodWiki::Runner::comment->{$page}) {
	    my $num = scalar keys %{$PodWiki::Runner::comment->{$page}};
	    my $T = "comment";
	    $T = "comments" if ($num > 1);
	    print qq( <a href="$PodWiki::Runner::me?page=$page#comment">$num $T</a>);
	}
        print "</div><br/><br/>\n";
      }
  }
 
  if (!$xml) {
    print qq(<p><a href="feed:http://$ENV{HTTP_HOST}$ENV{SCRIPT_NAME}?state=feed&blog=$PodWiki::Runner::page">RSS Feed for $PodWiki::Runner::page</a></p>);
  }
  
}

1;
